@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package consul
 
 import (
@@ -6,11 +9,13 @@ import (
 	"testing"
 	"time"
 
+	"github.com/stretchr/testify/require"
+
+	msgpackrpc "github.com/hashicorp/consul-net-rpc/net-rpc-msgpackrpc"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/sdk/testutil"
-	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
-	"github.com/stretchr/testify/require"
 )
 
 // Test basic creation
@@ -73,6 +78,8 @@ func TestIntentionApply_new(t *testing.T) {
 		actual.Hash = ixn.Intention.Hash
 		//nolint:staticcheck
 		ixn.Intention.UpdatePrecedence()
+		// Partition fields will be normalized on Intention.Get
+		ixn.Intention.FillPartitionAndNamespace(nil, true)
 		require.Equal(t, ixn.Intention, actual)
 	}
 
@@ -96,7 +103,7 @@ func TestIntentionApply_new(t *testing.T) {
 
 		var reply string
 		err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn2, &reply)
-		testutil.RequireErrorContains(t, err, "Cannot modify DestinationNS or DestinationName for an intention once it exists.")
+		testutil.RequireErrorContains(t, err, "Cannot modify Destination partition/namespace/name for an intention once it exists.")
 	})
 }
 
@@ -108,7 +115,6 @@ func TestIntentionApply_defaultSourceType(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -132,8 +138,8 @@ func TestIntentionApply_defaultSourceType(t *testing.T) {
 	var reply string
 
 	// Create
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
-	require.NotEmpty(reply)
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.NotEmpty(t, reply)
 
 	// Read
 	ixn.Intention.ID = reply
@@ -143,10 +149,10 @@ func TestIntentionApply_defaultSourceType(t *testing.T) {
 			IntentionID: ixn.Intention.ID,
 		}
 		var resp structs.IndexedIntentions
-		require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp))
-		require.Len(resp.Intentions, 1)
+		require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp))
+		require.Len(t, resp.Intentions, 1)
 		actual := resp.Intentions[0]
-		require.Equal(structs.IntentionSourceConsul, actual.SourceType)
+		require.Equal(t, structs.IntentionSourceConsul, actual.SourceType)
 	}
 }
 
@@ -158,7 +164,6 @@ func TestIntentionApply_createWithID(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -172,16 +177,17 @@ func TestIntentionApply_createWithID(t *testing.T) {
 		Datacenter: "dc1",
 		Op:         structs.IntentionOpCreate,
 		Intention: &structs.Intention{
-			ID:         generateUUID(),
-			SourceName: "test",
+			ID:              generateUUID(),
+			SourceName:      "test",
+			DestinationName: "test2",
 		},
 	}
 	var reply string
 
 	// Create
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.NotNil(err)
-	require.Contains(err, "ID must be empty")
+	require.NotNil(t, err)
+	require.Contains(t, err, "ID must be empty")
 }
 
 // Test basic updating
@@ -264,10 +270,47 @@ func TestIntentionApply_updateGood(t *testing.T) {
 		actual.Hash = ixn.Intention.Hash
 		//nolint:staticcheck
 		ixn.Intention.UpdatePrecedence()
+		// Partition fields will be normalized on Intention.Get
+		ixn.Intention.FillPartitionAndNamespace(nil, true)
 		require.Equal(t, ixn.Intention, actual)
 	}
 }
 
+// TestIntentionApply_NoSourcePeer makes sure that no intention is created with a SourcePeer since this is not supported
+func TestIntentionApply_NoSourcePeer(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	_, s1 := testServer(t)
+	codec := rpcClient(t, s1)
+
+	waitForLeaderEstablishment(t, s1)
+
+	// Setup a basic record to create
+	ixn := structs.IntentionRequest{
+		Datacenter: "dc1",
+		Op:         structs.IntentionOpCreate,
+		Intention: &structs.Intention{
+			SourceNS:        structs.IntentionDefaultNamespace,
+			SourceName:      "test",
+			SourcePeer:      "peer1",
+			DestinationNS:   structs.IntentionDefaultNamespace,
+			DestinationName: "test",
+			Action:          structs.IntentionActionAllow,
+			SourceType:      structs.IntentionSourceConsul,
+			Meta:            map[string]string{},
+		},
+	}
+	var reply string
+	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
+	require.Error(t, err)
+	require.Contains(t, err, "SourcePeer field is not supported on this endpoint. Use config entries instead")
+	require.Empty(t, reply)
+}
+
 // Shouldn't be able to update a non-existent intention
 func TestIntentionApply_updateNonExist(t *testing.T) {
 	if testing.Short() {
@@ -276,7 +319,6 @@ func TestIntentionApply_updateNonExist(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -298,8 +340,8 @@ func TestIntentionApply_updateNonExist(t *testing.T) {
 
 	// Create
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.NotNil(err)
-	require.Contains(err, "Cannot modify non-existent intention")
+	require.NotNil(t, err)
+	require.Contains(t, err, "Cannot modify non-existent intention")
 }
 
 // Test basic deleting
@@ -310,7 +352,6 @@ func TestIntentionApply_deleteGood(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -340,13 +381,13 @@ func TestIntentionApply_deleteGood(t *testing.T) {
 	}, &reply), "Cannot delete non-existent intention")
 
 	// Create
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
-	require.NotEmpty(reply)
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.NotEmpty(t, reply)
 
 	// Delete
 	ixn.Op = structs.IntentionOpDelete
 	ixn.Intention.ID = reply
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Read
 	ixn.Intention.ID = reply
@@ -357,8 +398,8 @@ func TestIntentionApply_deleteGood(t *testing.T) {
 		}
 		var resp structs.IndexedIntentions
 		err := msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp)
-		require.NotNil(err)
-		require.Contains(err, ErrIntentionNotFound.Error())
+		require.NotNil(t, err)
+		require.Contains(t, err, ErrIntentionNotFound.Error())
 	}
 }
 
@@ -377,7 +418,7 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	defaultEntMeta := structs.DefaultEnterpriseMeta()
+	defaultEntMeta := structs.DefaultEnterpriseMetaInDefaultPartition()
 
 	// Force "test" to be L7-capable.
 	{
@@ -413,7 +454,7 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 	opList := func() (*structs.IndexedIntentions, error) {
 		req := &structs.IntentionListRequest{
 			Datacenter:     "dc1",
-			EnterpriseMeta: *structs.WildcardEnterpriseMeta(),
+			EnterpriseMeta: *structs.WildcardEnterpriseMetaInDefaultPartition(),
 		}
 		var resp structs.IndexedIntentions
 		if err := msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp); err != nil {
@@ -510,7 +551,7 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 			},
 			RaftIndex: entry.RaftIndex,
 		}
-
+		entry.Hash = 0
 		require.Equal(t, expect, entry)
 	}
 
@@ -649,7 +690,7 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 			},
 			RaftIndex: entry.RaftIndex,
 		}
-
+		entry.Hash = 0
 		require.Equal(t, expect, entry)
 	}
 
@@ -718,7 +759,7 @@ func TestIntentionApply_WithoutIDs(t *testing.T) {
 			},
 			RaftIndex: entry.RaftIndex,
 		}
-
+		entry.Hash = 0
 		require.Equal(t, expect, entry)
 	}
 
@@ -857,12 +898,11 @@ func TestIntentionApply_aclDeny(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -871,27 +911,12 @@ func TestIntentionApply_aclDeny(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	// Create an ACL with write permissions
-	var token string
-	{
-		var rules = `
-service "foo" {
+	rules := `
+service "foobar" {
 	policy = "deny"
 	intentions = "write"
 }`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token := createToken(t, codec, rules)
 
 	// Setup a basic record to create
 	ixn := structs.IntentionRequest{
@@ -904,11 +929,11 @@ service "foo" {
 	// Create without a token should error since default deny
 	var reply string
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.True(acl.IsErrPermissionDenied(err))
+	require.True(t, acl.IsErrPermissionDenied(err))
 
 	// Now add the token and try again.
 	ixn.WriteRequest.Token = token
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Read
 	ixn.Intention.ID = reply
@@ -919,10 +944,10 @@ service "foo" {
 			QueryOptions: structs.QueryOptions{Token: "root"},
 		}
 		var resp structs.IndexedIntentions
-		require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp))
-		require.Len(resp.Intentions, 1)
+		require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp))
+		require.Len(t, resp.Intentions, 1)
 		actual := resp.Intentions[0]
-		require.Equal(resp.Index, actual.ModifyIndex)
+		require.Equal(t, resp.Index, actual.ModifyIndex)
 
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
 		actual.CreatedAt = ixn.Intention.CreatedAt
@@ -930,7 +955,7 @@ service "foo" {
 		actual.Hash = ixn.Intention.Hash
 		//nolint:staticcheck
 		ixn.Intention.UpdatePrecedence()
-		require.Equal(ixn.Intention, actual)
+		require.Equal(t, ixn.Intention, actual)
 	}
 }
 
@@ -946,17 +971,17 @@ func TestIntention_WildcardACLEnforcement(t *testing.T) {
 
 	// create some test policies.
 
-	writeToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "write" }`)
+	writeToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service_prefix "" { policy = "deny" intentions = "write" }`)
 	require.NoError(t, err)
-	readToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "read" }`)
+	readToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service_prefix "" { policy = "deny" intentions = "read" }`)
 	require.NoError(t, err)
-	exactToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "*" { policy = "deny" intentions = "write" }`)
+	exactToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service "*" { policy = "deny" intentions = "write" }`)
 	require.NoError(t, err)
-	wildcardPrefixToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "*" { policy = "deny" intentions = "write" }`)
+	wildcardPrefixToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service_prefix "*" { policy = "deny" intentions = "write" }`)
 	require.NoError(t, err)
-	fooToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "foo" { policy = "deny" intentions = "write" }`)
+	fooToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service "foo" { policy = "deny" intentions = "write" }`)
 	require.NoError(t, err)
-	denyToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "deny" }`)
+	denyToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service_prefix "" { policy = "deny" intentions = "deny" }`)
 	require.NoError(t, err)
 
 	doIntentionCreate := func(t *testing.T, token string, dest string, deny bool) string {
@@ -1262,12 +1287,11 @@ func TestIntentionApply_aclDelete(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1276,27 +1300,12 @@ func TestIntentionApply_aclDelete(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	// Create an ACL with write permissions
-	var token string
-	{
-		var rules = `
-service "foo" {
+	rules := `
+service "foobar" {
 	policy = "deny"
 	intentions = "write"
 }`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token := createToken(t, codec, rules)
 
 	// Setup a basic record to create
 	ixn := structs.IntentionRequest{
@@ -1309,18 +1318,18 @@ service "foo" {
 
 	// Create
 	var reply string
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Try to do a delete with no token; this should get rejected.
 	ixn.Op = structs.IntentionOpDelete
 	ixn.Intention.ID = reply
 	ixn.WriteRequest.Token = ""
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.True(acl.IsErrPermissionDenied(err))
+	require.True(t, acl.IsErrPermissionDenied(err))
 
 	// Try again with the original token. This should go through.
 	ixn.WriteRequest.Token = token
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Verify it is gone
 	{
@@ -1330,8 +1339,8 @@ service "foo" {
 		}
 		var resp structs.IndexedIntentions
 		err := msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp)
-		require.NotNil(err)
-		require.Contains(err.Error(), ErrIntentionNotFound.Error())
+		require.NotNil(t, err)
+		require.Contains(t, err.Error(), ErrIntentionNotFound.Error())
 	}
 }
 
@@ -1343,12 +1352,11 @@ func TestIntentionApply_aclUpdate(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1357,27 +1365,12 @@ func TestIntentionApply_aclUpdate(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	// Create an ACL with write permissions
-	var token string
-	{
-		var rules = `
-service "foo" {
+	rules := `
+service "foobar" {
 	policy = "deny"
 	intentions = "write"
 }`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token := createToken(t, codec, rules)
 
 	// Setup a basic record to create
 	ixn := structs.IntentionRequest{
@@ -1390,18 +1383,18 @@ service "foo" {
 
 	// Create
 	var reply string
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Try to do an update without a token; this should get rejected.
 	ixn.Op = structs.IntentionOpUpdate
 	ixn.Intention.ID = reply
 	ixn.WriteRequest.Token = ""
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.True(acl.IsErrPermissionDenied(err))
+	require.True(t, acl.IsErrPermissionDenied(err))
 
 	// Try again with the original token; this should go through.
 	ixn.WriteRequest.Token = token
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 }
 
 // Test apply with a management token
@@ -1412,12 +1405,11 @@ func TestIntentionApply_aclManagement(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1437,16 +1429,16 @@ func TestIntentionApply_aclManagement(t *testing.T) {
 
 	// Create
 	var reply string
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 	ixn.Intention.ID = reply
 
 	// Update
 	ixn.Op = structs.IntentionOpUpdate
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Delete
 	ixn.Op = structs.IntentionOpDelete
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 }
 
 // Test update changing the name where an ACL won't allow it
@@ -1457,12 +1449,11 @@ func TestIntentionApply_aclUpdateChange(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1471,27 +1462,12 @@ func TestIntentionApply_aclUpdateChange(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	// Create an ACL with write permissions
-	var token string
-	{
-		var rules = `
-service "foo" {
+	rules := `
+service "foobar" {
 	policy = "deny"
 	intentions = "write"
 }`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token := createToken(t, codec, rules)
 
 	// Setup a basic record to create
 	ixn := structs.IntentionRequest{
@@ -1504,7 +1480,7 @@ service "foo" {
 
 	// Create
 	var reply string
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+	require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 
 	// Try to do an update without a token; this should get rejected.
 	ixn.Op = structs.IntentionOpUpdate
@@ -1512,7 +1488,7 @@ service "foo" {
 	ixn.Intention.DestinationName = "foo"
 	ixn.WriteRequest.Token = token
 	err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
-	require.True(acl.IsErrPermissionDenied(err))
+	require.True(t, acl.IsErrPermissionDenied(err))
 }
 
 // Test reading with ACLs
@@ -1524,10 +1500,10 @@ func TestIntentionGet_acl(t *testing.T) {
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1624,7 +1600,6 @@ func TestIntentionList(t *testing.T) {
 
 	t.Parallel()
 
-	require := require.New(t)
 	dir1, s1 := testServer(t)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -1639,9 +1614,9 @@ func TestIntentionList(t *testing.T) {
 			Datacenter: "dc1",
 		}
 		var resp structs.IndexedIntentions
-		require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
-		require.NotNil(resp.Intentions)
-		require.Len(resp.Intentions, 0)
+		require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
+		require.NotNil(t, resp.Intentions)
+		require.Len(t, resp.Intentions, 0)
 	}
 }
 
@@ -1653,16 +1628,15 @@ func TestIntentionList_acl(t *testing.T) {
 
 	t.Parallel()
 
-	dir1, s1 := testServerWithConfig(t, testServerACLConfig(nil))
+	dir1, s1 := testServerWithConfig(t, testServerACLConfig)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
 	waitForLeaderEstablishment(t, s1)
-	waitForNewACLs(t, s1)
 
-	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "foo" { policy = "write" }`)
+	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service_prefix "foo" { policy = "write" }`)
 	require.NoError(t, err)
 
 	// Create a few records
@@ -1675,7 +1649,7 @@ func TestIntentionList_acl(t *testing.T) {
 		ixn.Intention.SourceNS = "default"
 		ixn.Intention.DestinationNS = "default"
 		ixn.Intention.DestinationName = name
-		ixn.WriteRequest.Token = TestDefaultMasterToken
+		ixn.WriteRequest.Token = TestDefaultInitialManagementToken
 
 		// Create
 		var reply string
@@ -1690,17 +1664,19 @@ func TestIntentionList_acl(t *testing.T) {
 		var resp structs.IndexedIntentions
 		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
 		require.Len(t, resp.Intentions, 0)
+		require.False(t, resp.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
 	})
 
 	// Test with management token
-	t.Run("master-token", func(t *testing.T) {
+	t.Run("initial-management-token", func(t *testing.T) {
 		req := &structs.IntentionListRequest{
 			Datacenter:   "dc1",
-			QueryOptions: structs.QueryOptions{Token: TestDefaultMasterToken},
+			QueryOptions: structs.QueryOptions{Token: TestDefaultInitialManagementToken},
 		}
 		var resp structs.IndexedIntentions
 		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
 		require.Len(t, resp.Intentions, 3)
+		require.False(t, resp.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
 	})
 
 	// Test with user token
@@ -1712,13 +1688,14 @@ func TestIntentionList_acl(t *testing.T) {
 		var resp structs.IndexedIntentions
 		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
 		require.Len(t, resp.Intentions, 1)
+		require.True(t, resp.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be true")
 	})
 
 	t.Run("filtered", func(t *testing.T) {
 		req := &structs.IntentionListRequest{
 			Datacenter: "dc1",
 			QueryOptions: structs.QueryOptions{
-				Token:  TestDefaultMasterToken,
+				Token:  TestDefaultInitialManagementToken,
 				Filter: "DestinationName == foobar",
 			},
 		}
@@ -1726,6 +1703,7 @@ func TestIntentionList_acl(t *testing.T) {
 		var resp structs.IndexedIntentions
 		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp))
 		require.Len(t, resp.Intentions, 1)
+		require.False(t, resp.QueryMeta.ResultsFilteredByACLs, "ResultsFilteredByACLs should be false")
 	})
 }
 
@@ -1803,6 +1781,98 @@ func TestIntentionMatch_good(t *testing.T) {
 	require.Equal(t, expected, actual)
 }
 
+func TestIntentionMatch_BlockOnNoChange(t *testing.T) {
+	if testing.Short() {
+		t.Skip("too slow for testing.Short")
+	}
+
+	t.Parallel()
+
+	_, s1 := testServerWithConfig(t, func(c *Config) {
+		c.DevMode = true // keep it in ram to make it 10x faster on macos
+	})
+
+	codec := rpcClient(t, s1)
+
+	waitForLeaderEstablishment(t, s1)
+
+	run := func(t *testing.T, dataPrefix string, expectMatches int) {
+		rpcBlockingQueryTestHarness(t,
+			func(minQueryIndex uint64) (*structs.QueryMeta, <-chan error) {
+				args := &structs.IntentionQueryRequest{
+					Datacenter: "dc1",
+					Match: &structs.IntentionQueryMatch{
+						Type: structs.IntentionMatchDestination,
+						Entries: []structs.IntentionMatchEntry{
+							{Name: "bar"},
+						},
+					},
+				}
+				args.QueryOptions.MinQueryIndex = minQueryIndex
+
+				var out structs.IndexedIntentionMatches
+				errCh := channelCallRPC(s1, "Intention.Match", args, &out, func() error {
+					if len(out.Matches) != 1 {
+						return fmt.Errorf("expected 1 match got %d", len(out.Matches))
+					}
+					if len(out.Matches[0]) != expectMatches {
+						return fmt.Errorf("expected %d inner matches got %d", expectMatches, len(out.Matches[0]))
+					}
+					return nil
+				})
+				return &out.QueryMeta, errCh
+			},
+			func(i int) <-chan error {
+				var out string
+				return channelCallRPC(s1, "Intention.Apply", &structs.IntentionRequest{
+					Datacenter: "dc1",
+					Op:         structs.IntentionOpCreate,
+					Intention: &structs.Intention{
+						// {"default", "*", "default", "baz"}, // shouldn't match
+						SourceNS:        "default",
+						SourceName:      "*",
+						DestinationNS:   "default",
+						DestinationName: fmt.Sprintf(dataPrefix+"%d", i),
+						Action:          structs.IntentionActionAllow,
+					},
+				}, &out, nil)
+			},
+		)
+	}
+
+	testutil.RunStep(t, "test the errNotFound path", func(t *testing.T) {
+		run(t, "other", 0)
+	})
+
+	// Create some records
+	{
+		insert := [][]string{
+			{"default", "*", "default", "*"},
+			{"default", "*", "default", "bar"},
+			{"default", "*", "default", "baz"}, // shouldn't match
+		}
+
+		for _, v := range insert {
+			var out string
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &structs.IntentionRequest{
+				Datacenter: "dc1",
+				Op:         structs.IntentionOpCreate,
+				Intention: &structs.Intention{
+					SourceNS:        v[0],
+					SourceName:      v[1],
+					DestinationNS:   v[2],
+					DestinationName: v[3],
+					Action:          structs.IntentionActionAllow,
+				},
+			}, &out))
+		}
+	}
+
+	testutil.RunStep(t, "test the errNotChanged path", func(t *testing.T) {
+		run(t, "completely-different-other", 2)
+	})
+}
+
 // Test matching with ACLs
 func TestIntentionMatch_acl(t *testing.T) {
 	if testing.Short() {
@@ -1814,7 +1884,7 @@ func TestIntentionMatch_acl(t *testing.T) {
 	_, srv, codec := testACLServerWithConfig(t, nil, false)
 	waitForLeaderEstablishment(t, srv)
 
-	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "bar" { policy = "write" }`)
+	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service "bar" { policy = "write" }`)
 	require.NoError(t, err)
 
 	// Create some records
@@ -1832,7 +1902,7 @@ func TestIntentionMatch_acl(t *testing.T) {
 				Intention:  structs.TestIntention(t),
 			}
 			ixn.Intention.DestinationName = v
-			ixn.WriteRequest.Token = TestDefaultMasterToken
+			ixn.WriteRequest.Token = TestDefaultInitialManagementToken
 
 			// Create
 			var reply string
@@ -1856,6 +1926,7 @@ func TestIntentionMatch_acl(t *testing.T) {
 		}
 		var resp structs.IndexedIntentionMatches
 		err := msgpackrpc.CallWithCodec(codec, "Intention.Match", req, &resp)
+		require.Error(t, err)
 		require.True(t, acl.IsErrPermissionDenied(err))
 		require.Len(t, resp.Matches, 0)
 	}
@@ -1889,106 +1960,89 @@ func TestIntentionMatch_acl(t *testing.T) {
 	}
 }
 
-// Test the Check method defaults to allow with no ACL set.
-func TestIntentionCheck_defaultNoACL(t *testing.T) {
+func TestIntentionCheck(t *testing.T) {
 	if testing.Short() {
 		t.Skip("too slow for testing.Short")
 	}
 
 	t.Parallel()
 
-	dir1, s1 := testServer(t)
-	defer os.RemoveAll(dir1)
-	defer s1.Shutdown()
-	codec := rpcClient(t, s1)
-	defer codec.Close()
-
-	waitForLeaderEstablishment(t, s1)
-
-	// Test
-	req := &structs.IntentionQueryRequest{
-		Datacenter: "dc1",
-		Check: &structs.IntentionQueryCheck{
-			SourceName:      "bar",
-			DestinationName: "qux",
-			SourceType:      structs.IntentionSourceConsul,
+	type testcase struct {
+		aclsEnabled   bool
+		defaultACL    string
+		defaultIxn    string
+		expectAllowed bool
+	}
+	tcs := map[string]testcase{
+		"acls disabled, no default intention policy": {
+			aclsEnabled:   false,
+			expectAllowed: true,
+		},
+		"acls disabled, default intention allow": {
+			aclsEnabled:   false,
+			defaultIxn:    "allow",
+			expectAllowed: true,
+		},
+		"acls disabled, default intention deny": {
+			aclsEnabled:   false,
+			defaultIxn:    "deny",
+			expectAllowed: false,
+		},
+		"acls deny, no default intention policy": {
+			aclsEnabled:   true,
+			defaultACL:    "deny",
+			expectAllowed: false,
+		},
+		"acls allow, no default intention policy": {
+			aclsEnabled:   true,
+			defaultACL:    "allow",
+			expectAllowed: true,
+		},
+		"acls deny, default intention allow": {
+			aclsEnabled:   true,
+			defaultACL:    "deny",
+			defaultIxn:    "allow",
+			expectAllowed: true,
+		},
+		"acls allow, default intention deny": {
+			aclsEnabled:   true,
+			defaultACL:    "allow",
+			defaultIxn:    "deny",
+			expectAllowed: false,
 		},
 	}
-	var resp structs.IntentionQueryCheckResponse
-	require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
-	require.True(t, resp.Allowed)
-}
+	for name, tc := range tcs {
+		tc := tc
+		t.Run(name, func(t *testing.T) {
+			t.Parallel()
+			_, s1 := testServerWithConfig(t, func(c *Config) {
+				if tc.aclsEnabled {
+					c.PrimaryDatacenter = "dc1"
+					c.ACLsEnabled = true
+					c.ACLInitialManagementToken = "root"
+					c.ACLResolverSettings.ACLDefaultPolicy = tc.defaultACL
+				}
+				c.DefaultIntentionPolicy = tc.defaultIxn
+			})
+			codec := rpcClient(t, s1)
 
-// Test the Check method defaults to deny with allowlist ACLs.
-func TestIntentionCheck_defaultACLDeny(t *testing.T) {
-	if testing.Short() {
-		t.Skip("too slow for testing.Short")
+			waitForLeaderEstablishment(t, s1)
+
+			req := &structs.IntentionQueryRequest{
+				Datacenter: "dc1",
+				Check: &structs.IntentionQueryCheck{
+					SourceName:      "bar",
+					DestinationName: "qux",
+					SourceType:      structs.IntentionSourceConsul,
+				},
+			}
+			req.Token = "root"
+
+			var resp structs.IntentionQueryCheckResponse
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
+			require.Equal(t, tc.expectAllowed, resp.Allowed)
+		})
 	}
-
-	t.Parallel()
-
-	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-	})
-	defer os.RemoveAll(dir1)
-	defer s1.Shutdown()
-	codec := rpcClient(t, s1)
-	defer codec.Close()
-
-	waitForLeaderEstablishment(t, s1)
-
-	// Check
-	req := &structs.IntentionQueryRequest{
-		Datacenter: "dc1",
-		Check: &structs.IntentionQueryCheck{
-			SourceName:      "bar",
-			DestinationName: "qux",
-			SourceType:      structs.IntentionSourceConsul,
-		},
-	}
-	req.Token = "root"
-	var resp structs.IntentionQueryCheckResponse
-	require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
-	require.False(t, resp.Allowed)
-}
-
-// Test the Check method defaults to deny with denylist ACLs.
-func TestIntentionCheck_defaultACLAllow(t *testing.T) {
-	if testing.Short() {
-		t.Skip("too slow for testing.Short")
-	}
-
-	t.Parallel()
-
-	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "allow"
-	})
-	defer os.RemoveAll(dir1)
-	defer s1.Shutdown()
-	codec := rpcClient(t, s1)
-	defer codec.Close()
-
-	waitForLeaderEstablishment(t, s1)
-
-	// Check
-	req := &structs.IntentionQueryRequest{
-		Datacenter: "dc1",
-		Check: &structs.IntentionQueryCheck{
-			SourceName:      "bar",
-			DestinationName: "qux",
-			SourceType:      structs.IntentionSourceConsul,
-		},
-	}
-	req.Token = "root"
-	var resp structs.IntentionQueryCheckResponse
-	require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
-	require.True(t, resp.Allowed)
 }
 
 // Test the Check method requires service:read permission.
@@ -2000,10 +2054,10 @@ func TestIntentionCheck_aclDeny(t *testing.T) {
 	t.Parallel()
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
@@ -2012,26 +2066,11 @@ func TestIntentionCheck_aclDeny(t *testing.T) {
 
 	waitForLeaderEstablishment(t, s1)
 
-	// Create an ACL with service read permissions. This will grant permission.
-	var token string
-	{
-		var rules = `
+	rules := `
 service "bar" {
 	policy = "read"
 }`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.NoError(t, msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token := createToken(t, codec, rules)
 
 	// Check
 	req := &structs.IntentionQueryRequest{
@@ -2059,7 +2098,7 @@ func TestIntentionCheck_match(t *testing.T) {
 	_, srv, codec := testACLServerWithConfig(t, nil, false)
 	waitForLeaderEstablishment(t, srv)
 
-	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "api" { policy = "read" }`)
+	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultInitialManagementToken, "dc1", `service "api" { policy = "read" }`)
 	require.NoError(t, err)
 
 	// Create some intentions
@@ -2081,7 +2120,7 @@ func TestIntentionCheck_match(t *testing.T) {
 					DestinationName: v[1],
 					Action:          structs.IntentionActionAllow,
 				},
-				WriteRequest: structs.WriteRequest{Token: TestDefaultMasterToken},
+				WriteRequest: structs.WriteRequest{Token: TestDefaultInitialManagementToken},
 			}
 			// Create
 			var reply string
