@@ -4,14 +4,10 @@
 package consul
 
 import (
-	"github.com/hashicorp/consul/internal/auth"
-	"github.com/hashicorp/consul/internal/catalog"
 	"github.com/hashicorp/consul/internal/hcp"
-	"github.com/hashicorp/consul/internal/mesh"
 	"github.com/hashicorp/consul/internal/multicluster"
 	"github.com/hashicorp/consul/internal/resource"
 	"github.com/hashicorp/consul/internal/resource/demo"
-	"github.com/hashicorp/consul/internal/tenancy"
 )
 
 // NewTypeRegistry returns a registry populated with all supported resource
@@ -25,10 +21,6 @@ func NewTypeRegistry() resource.Registry {
 	registry := resource.NewRegistry()
 
 	demo.RegisterTypes(registry)
-	mesh.RegisterTypes(registry)
-	catalog.RegisterTypes(registry)
-	auth.RegisterTypes(registry)
-	tenancy.RegisterTypes(registry)
 	multicluster.RegisterTypes(registry)
 	hcp.RegisterTypes(registry)
 
