@@ -14,7 +14,7 @@ import (
 	"github.com/hashicorp/consul/acl/resolver"
 	"github.com/hashicorp/consul/agent/consul"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/internal/gossip/librtt"
 	"github.com/hashicorp/consul/proto-public/pbresource"
 )
 
@@ -22,9 +22,9 @@ type delegateMock struct {
 	mock.Mock
 }
 
-func (m *delegateMock) GetLANCoordinate() (lib.CoordinateSet, error) {
+func (m *delegateMock) GetLANCoordinate() (librtt.CoordinateSet, error) {
 	ret := m.Called()
-	return ret.Get(0).(lib.CoordinateSet), ret.Error(1)
+	return ret.Get(0).(librtt.CoordinateSet), ret.Error(1)
 }
 
 func (m *delegateMock) Leave() error {
