@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package proxy
 
 import (
@@ -27,8 +30,7 @@ func TestProxy_public(t *testing.T) {
 		t.Skip("too slow for testing.Short")
 	}
 
-	ports := freeport.MustTake(2)
-	defer freeport.Return(ports)
+	ports := freeport.GetN(t, 2)
 
 	a := agent.NewTestAgent(t, "")
 	defer a.Shutdown()
@@ -96,7 +98,7 @@ func TestProxy_public(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		conn, err = svc.Dial(context.Background(), &connect.StaticResolver{
 			Addr:    TestLocalAddr(ports[0]),
-			CertURI: agConnect.TestSpiffeIDService(t, "echo"),
+			CertURI: agConnect.TestSpiffeIDService(r, "echo"),
 		})
 		if err != nil {
 			r.Fatalf("err: %s", err)
