@@ -13,32 +13,39 @@ import (
 	"sort"
 
 	"github.com/google/go-cmp/cmp"
-	pbauth "github.com/hashicorp/consul/proto-public/pbauth/v2beta1"
-	pbcatalog "github.com/hashicorp/consul/proto-public/pbcatalog/v2beta1"
-	pbmesh "github.com/hashicorp/consul/proto-public/pbmesh/v2beta1"
-	"github.com/hashicorp/consul/proto-public/pbresource"
-	"github.com/hashicorp/go-hclog"
 	"golang.org/x/exp/maps"
 
+	"github.com/hashicorp/go-hclog"
+
+	"github.com/hashicorp/consul/proto-public/pbresource"
 	"github.com/hashicorp/consul/testing/deployer/util"
 )
 
 const DockerPrefix = "cslc" // ConSuLCluster
 
 func Compile(logger hclog.Logger, raw *Config) (*Topology, error) {
-	return compile(logger, raw, nil)
+	return compile(logger, raw, nil, "")
 }
 
 func Recompile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error) {
 	if prev == nil {
 		return nil, errors.New("missing previous topology")
 	}
-	return compile(logger, raw, prev)
+	return compile(logger, raw, prev, "")
 }
 
-func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error) {
+func compile(logger hclog.Logger, raw *Config, prev *Topology, testingID string) (*Topology, error) {
+	if logger == nil {
+		return nil, errors.New("logger is required")
+	}
+	if raw == nil {
+		return nil, errors.New("config is required")
+	}
+
 	var id string
-	if prev == nil {
+	if testingID != "" {
+		id = testingID
+	} else if prev == nil {
 		var err error
 		id, err = newTopologyID()
 		if err != nil {
@@ -90,22 +97,17 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 		nextIndex int // use a global index so any shared networks work properly with assignments
 	)
 
-	foundPeerNames := make(map[string]map[string]struct{})
-	for _, c := range raw.Clusters {
+	compileCluster := func(c *Cluster) (map[string]struct{}, error) {
 		if c.Name == "" {
 			return nil, fmt.Errorf("cluster has no name")
 		}
 
-		foundPeerNames[c.Name] = make(map[string]struct{})
+		foundPeerNames := make(map[string]struct{})
 
 		if !IsValidLabel(c.Name) {
 			return nil, fmt.Errorf("cluster name is not valid: %s", c.Name)
 		}
 
-		if _, exists := clusters[c.Name]; exists {
-			return nil, fmt.Errorf("cannot have two clusters with the same name %q; use unique names and override the Datacenter field if that's what you want", c.Name)
-		}
-
 		if c.Datacenter == "" {
 			c.Datacenter = c.Name
 		} else {
@@ -114,7 +116,6 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 			}
 		}
 
-		clusters[c.Name] = c
 		if c.NetworkName == "" {
 			c.NetworkName = c.Name
 		}
@@ -129,22 +130,6 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 			return nil, fmt.Errorf("cluster %q has no nodes", c.Name)
 		}
 
-		if len(c.Services) == 0 { // always initialize this regardless of v2-ness, because we might late-enable it below
-			c.Services = make(map[ID]*pbcatalog.Service)
-		}
-
-		var implicitV2Services bool
-		if len(c.Services) > 0 {
-			c.EnableV2 = true
-			for name, svc := range c.Services {
-				if svc.Workloads != nil {
-					return nil, fmt.Errorf("the workloads field for v2 service %q is not user settable", name)
-				}
-			}
-		} else {
-			implicitV2Services = true
-		}
-
 		if c.TLSVolumeName != "" {
 			return nil, fmt.Errorf("user cannot specify the TLSVolumeName field")
 		}
@@ -158,6 +143,7 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				m = make(map[string]struct{})
 				tenancies[partition] = m
 			}
+			m["default"] = struct{}{}
 			m[namespace] = struct{}{}
 		}
 
@@ -172,39 +158,24 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 			addTenancy(ce.GetPartition(), ce.GetNamespace())
 		}
 
-		if len(c.InitialResources) > 0 {
-			c.EnableV2 = true
-		}
 		for _, res := range c.InitialResources {
 			if res.Id.Tenancy == nil {
 				res.Id.Tenancy = &pbresource.Tenancy{}
 			}
-			// TODO(peering/v2) prevent non-local peer resources
 			res.Id.Tenancy.Partition = PartitionOrDefault(res.Id.Tenancy.Partition)
 			if !util.IsTypePartitionScoped(res.Id.Type) {
 				res.Id.Tenancy.Namespace = NamespaceOrDefault(res.Id.Tenancy.Namespace)
 			}
 
-			switch {
-			case util.EqualType(pbauth.ComputedTrafficPermissionsType, res.Id.GetType()),
-				util.EqualType(pbauth.WorkloadIdentityType, res.Id.GetType()):
-				fallthrough
-			case util.EqualType(pbmesh.ComputedRoutesType, res.Id.GetType()),
-				util.EqualType(pbmesh.ProxyStateTemplateType, res.Id.GetType()):
-				fallthrough
-			case util.EqualType(pbcatalog.HealthChecksType, res.Id.GetType()),
-				util.EqualType(pbcatalog.HealthStatusType, res.Id.GetType()),
-				util.EqualType(pbcatalog.NodeType, res.Id.GetType()),
-				util.EqualType(pbcatalog.ServiceEndpointsType, res.Id.GetType()),
-				util.EqualType(pbcatalog.WorkloadType, res.Id.GetType()):
+			// TODO: if we reintroduce new resources for v1, allow them here
+			if true {
 				return nil, fmt.Errorf("you should not create a resource of type %q this way", util.TypeToString(res.Id.Type))
 			}
 
 			addTenancy(res.Id.Tenancy.Partition, res.Id.Tenancy.Namespace)
 		}
 
-		seenNodes := make(map[NodeID]struct{})
-		for _, n := range c.Nodes {
+		compileClusterNode := func(n *Node) (map[string]struct{}, error) {
 			if n.Name == "" {
 				return nil, fmt.Errorf("cluster %q node has no name", c.Name)
 			}
@@ -218,30 +189,15 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				return nil, fmt.Errorf("cluster %q node %q has invalid kind: %s", c.Name, n.Name, n.Kind)
 			}
 
-			if n.Version == NodeVersionUnknown {
-				n.Version = NodeVersionV1
-			}
-			switch n.Version {
-			case NodeVersionV1:
-			case NodeVersionV2:
-				if n.Kind == NodeKindClient {
-					return nil, fmt.Errorf("v2 does not support client agents at this time")
-				}
-				c.EnableV2 = true
-			default:
-				return nil, fmt.Errorf("cluster %q node %q has invalid version: %s", c.Name, n.Name, n.Version)
-			}
-
 			n.Partition = PartitionOrDefault(n.Partition)
 			if !IsValidLabel(n.Partition) {
 				return nil, fmt.Errorf("node partition is not valid: %s", n.Partition)
 			}
 			addTenancy(n.Partition, "default")
 
-			if _, exists := seenNodes[n.ID()]; exists {
-				return nil, fmt.Errorf("cannot have two nodes in the same cluster %q with the same name %q", c.Name, n.ID())
+			if n.IsServer() && n.Partition != "default" {
+				return nil, fmt.Errorf("server agents must always be in the default partition")
 			}
-			seenNodes[n.ID()] = struct{}{}
 
 			if len(n.usedPorts) != 0 {
 				return nil, fmt.Errorf("user cannot specify the usedPorts field")
@@ -315,12 +271,6 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				return nil, fmt.Errorf("cluster %q node %q has more than one public address", c.Name, n.Name)
 			}
 
-			if len(n.Services) > 0 {
-				logger.Warn("please use Node.Workloads instead of Node.Services")
-				n.Workloads = append(n.Workloads, n.Services...)
-				n.Services = nil
-			}
-
 			if n.IsDataplane() && len(n.Workloads) > 1 {
 				// Our use of consul-dataplane here is supposed to mimic that
 				// of consul-k8s, which ultimately has one IP per Service, so
@@ -328,7 +278,10 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				return nil, fmt.Errorf("cluster %q node %q uses dataplane, but has more than one service", c.Name, n.Name)
 			}
 
-			seenServices := make(map[ID]struct{})
+			var (
+				foundPeerNames = make(map[string]struct{})
+				seenServices   = make(map[ID]struct{})
+			)
 			for _, wrk := range n.Workloads {
 				if n.IsAgent() {
 					// Default to that of the enclosing node.
@@ -338,10 +291,6 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 
 				// Denormalize
 				wrk.Node = n
-				wrk.NodeVersion = n.Version
-				if n.IsV2() {
-					wrk.Workload = wrk.ID.Name + "-" + n.Name
-				}
 
 				if !IsValidLabel(wrk.ID.Partition) {
 					return nil, fmt.Errorf("service partition is not valid: %s", wrk.ID.Partition)
@@ -398,166 +347,59 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				// 	return nil, fmt.Errorf("service has invalid protocol: %s", wrk.Protocol)
 				// }
 
-				defaultDestination := func(dest *Destination) error {
+				defaultUpstream := func(us *Upstream) error {
 					// Default to that of the enclosing service.
-					if dest.Peer == "" {
-						if dest.ID.Partition == "" {
-							dest.ID.Partition = wrk.ID.Partition
+					if us.Peer == "" {
+						if us.ID.Partition == "" {
+							us.ID.Partition = wrk.ID.Partition
 						}
-						if dest.ID.Namespace == "" {
-							dest.ID.Namespace = wrk.ID.Namespace
+						if us.ID.Namespace == "" {
+							us.ID.Namespace = wrk.ID.Namespace
 						}
 					} else {
-						if dest.ID.Partition != "" {
-							dest.ID.Partition = "" // irrelevant here; we'll set it to the value of the OTHER side for plumbing purposes in tests
+						if us.ID.Partition != "" {
+							us.ID.Partition = "" // irrelevant here; we'll set it to the value of the OTHER side for plumbing purposes in tests
 						}
-						dest.ID.Namespace = NamespaceOrDefault(dest.ID.Namespace)
-						foundPeerNames[c.Name][dest.Peer] = struct{}{}
+						us.ID.Namespace = NamespaceOrDefault(us.ID.Namespace)
+						foundPeerNames[us.Peer] = struct{}{}
 					}
 
-					addTenancy(dest.ID.Partition, dest.ID.Namespace)
+					addTenancy(us.ID.Partition, us.ID.Namespace)
 
-					if dest.Implied {
-						if dest.PortName == "" {
-							return fmt.Errorf("implicit destinations must use port names in v2")
-						}
-					} else {
-						if dest.LocalAddress == "" {
-							// v1 defaults to 127.0.0.1 but v2 does not. Safe to do this generally though.
-							dest.LocalAddress = "127.0.0.1"
-						}
-						if dest.PortName != "" && n.IsV1() {
-							return fmt.Errorf("explicit destinations cannot use port names in v1")
-						}
-						if dest.PortName == "" && n.IsV2() {
-							// Assume this is a v1->v2 conversion and name it.
-							dest.PortName = "legacy"
-						}
+					if us.LocalAddress == "" {
+						// v1 consul code defaults this to 127.0.0.1, but safer to not rely upon that.
+						us.LocalAddress = "127.0.0.1"
 					}
 
 					return nil
 				}
 
-				for _, dest := range wrk.Destinations {
-					if err := defaultDestination(dest); err != nil {
+				for _, us := range wrk.Upstreams {
+					if err := defaultUpstream(us); err != nil {
 						return nil, err
 					}
 				}
 
-				if n.IsV2() {
-					for _, dest := range wrk.ImpliedDestinations {
-						dest.Implied = true
-						if err := defaultDestination(dest); err != nil {
-							return nil, err
-						}
-					}
-				} else {
-					if len(wrk.ImpliedDestinations) > 0 {
-						return nil, fmt.Errorf("v1 does not support implied destinations yet")
-					}
-				}
-
 				if err := wrk.Validate(); err != nil {
 					return nil, fmt.Errorf("cluster %q node %q service %q is not valid: %w", c.Name, n.Name, wrk.ID.String(), err)
 				}
-
-				if wrk.EnableTransparentProxy && !n.IsDataplane() {
-					return nil, fmt.Errorf("cannot enable tproxy on a non-dataplane node")
-				}
-
-				if n.IsV2() {
-					if implicitV2Services {
-						wrk.V2Services = []string{wrk.ID.Name}
-
-						var svcPorts []*pbcatalog.ServicePort
-						for name, cfg := range wrk.Ports {
-							svcPorts = append(svcPorts, &pbcatalog.ServicePort{
-								TargetPort: name,
-								Protocol:   cfg.ActualProtocol,
-							})
-						}
-
-						v2svc := &pbcatalog.Service{
-							Workloads: &pbcatalog.WorkloadSelector{},
-							Ports:     svcPorts,
-						}
-
-						prev, ok := c.Services[wrk.ID]
-						if !ok {
-							c.Services[wrk.ID] = v2svc
-							prev = v2svc
-						}
-						if prev.Workloads == nil {
-							prev.Workloads = &pbcatalog.WorkloadSelector{}
-						}
-						prev.Workloads.Names = append(prev.Workloads.Names, wrk.Workload)
-
-					} else {
-						for _, name := range wrk.V2Services {
-							v2ID := NewServiceID(name, wrk.ID.Namespace, wrk.ID.Partition)
-
-							v2svc, ok := c.Services[v2ID]
-							if !ok {
-								return nil, fmt.Errorf("cluster %q node %q service %q has a v2 service reference that does not exist %q",
-									c.Name, n.Name, wrk.ID.String(), name)
-							}
-							if v2svc.Workloads == nil {
-								v2svc.Workloads = &pbcatalog.WorkloadSelector{}
-							}
-							v2svc.Workloads.Names = append(v2svc.Workloads.Names, wrk.Workload)
-						}
-					}
-
-					if wrk.WorkloadIdentity == "" {
-						wrk.WorkloadIdentity = wrk.ID.Name
-					}
-				} else {
-					if len(wrk.V2Services) > 0 {
-						return nil, fmt.Errorf("cannot specify v2 services for v1")
-					}
-					if wrk.WorkloadIdentity != "" {
-						return nil, fmt.Errorf("cannot specify workload identities for v1")
-					}
-				}
 			}
+			return foundPeerNames, nil
 		}
 
-		if err := assignVirtualIPs(c); err != nil {
-			return nil, err
-		}
-
-		if c.EnableV2 {
-			// Populate the VirtualPort field on all destinations.
-			for _, n := range c.Nodes {
-				for _, wrk := range n.Workloads {
-					for _, dest := range wrk.ImpliedDestinations {
-						res, ok := c.Services[dest.ID]
-						if ok {
-							for _, sp := range res.Ports {
-								if sp.Protocol == pbcatalog.Protocol_PROTOCOL_MESH {
-									continue
-								}
-								if sp.MatchesPortId(dest.PortName) {
-									dest.VirtualPort = sp.VirtualPort
-								}
-							}
-						}
-					}
-					for _, dest := range wrk.Destinations {
-						res, ok := c.Services[dest.ID]
-						if ok {
-							for _, sp := range res.Ports {
-								if sp.Protocol == pbcatalog.Protocol_PROTOCOL_MESH {
-									continue
-								}
-								if sp.MatchesPortId(dest.PortName) {
-									dest.VirtualPort = sp.VirtualPort
-								}
-							}
-						}
-					}
-				}
+		seenNodes := make(map[NodeID]struct{})
+		for _, n := range c.Nodes {
+			peerNames, err := compileClusterNode(n)
+			if err != nil {
+				return nil, fmt.Errorf("error compiling node %q: %w", n.Name, err)
 			}
+
+			if _, exists := seenNodes[n.ID()]; exists {
+				return nil, fmt.Errorf("cannot have two nodes in the same cluster %q with the same name %q", c.Name, n.ID())
+			}
+			seenNodes[n.ID()] = struct{}{}
+
+			maps.Copy(foundPeerNames, peerNames)
 		}
 
 		// Explode this into the explicit list based on stray references made.
@@ -582,6 +424,24 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 				return nil, fmt.Errorf("cluster %q references non-default partitions or namespaces but is CE", c.Name)
 			}
 		}
+
+		return foundPeerNames, nil
+	}
+
+	foundPeerNames := make(map[string]map[string]struct{})
+	for _, c := range raw.Clusters {
+		peerNames, err := compileCluster(c)
+		if err != nil {
+			return nil, fmt.Errorf("error building cluster %q: %w", c.Name, err)
+		}
+
+		foundPeerNames[c.Name] = peerNames
+
+		if _, exists := clusters[c.Name]; exists {
+			return nil, fmt.Errorf("cannot have two clusters with the same name %q; use unique names and override the Datacenter field if that's what you want", c.Name)
+		}
+
+		clusters[c.Name] = c
 	}
 
 	clusteredPeerings := make(map[string]map[string]*PeerCluster) // local-cluster -> local-peer -> info
@@ -665,40 +525,25 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 		}
 	}
 
-	// after we decoded the peering stuff, we can fill in some computed data in the destinations
+	// after we decoded the peering stuff, we can fill in some computed data in the upstreams
 	for _, c := range clusters {
 		c.Peerings = clusteredPeerings[c.Name]
 		for _, n := range c.Nodes {
 			for _, wrk := range n.Workloads {
-				for _, dest := range wrk.Destinations {
-					if dest.Peer == "" {
-						dest.Cluster = c.Name
-						dest.Peering = nil
+				for _, us := range wrk.Upstreams {
+					if us.Peer == "" {
+						us.Cluster = c.Name
+						us.Peering = nil
 						continue
 					}
-					remotePeer, ok := c.Peerings[dest.Peer]
+					remotePeer, ok := c.Peerings[us.Peer]
 					if !ok {
 						return nil, fmt.Errorf("not possible")
 					}
-					dest.Cluster = remotePeer.Link.Name
-					dest.Peering = remotePeer.Link
+					us.Cluster = remotePeer.Link.Name
+					us.Peering = remotePeer.Link
 					// this helps in generating fortio assertions; otherwise field is ignored
-					dest.ID.Partition = remotePeer.Link.Partition
-				}
-				for _, dest := range wrk.ImpliedDestinations {
-					if dest.Peer == "" {
-						dest.Cluster = c.Name
-						dest.Peering = nil
-						continue
-					}
-					remotePeer, ok := c.Peerings[dest.Peer]
-					if !ok {
-						return nil, fmt.Errorf("not possible")
-					}
-					dest.Cluster = remotePeer.Link.Name
-					dest.Peering = remotePeer.Link
-					// this helps in generating fortio assertions; otherwise field is ignored
-					dest.ID.Partition = remotePeer.Link.Partition
+					us.ID.Partition = remotePeer.Link.Partition
 				}
 			}
 		}
@@ -767,51 +612,6 @@ func compile(logger hclog.Logger, raw *Config, prev *Topology) (*Topology, error
 	return t, nil
 }
 
-func assignVirtualIPs(c *Cluster) error {
-	lastVIPIndex := 1
-	for _, svcData := range c.Services {
-		lastVIPIndex++
-		if lastVIPIndex > 250 {
-			return fmt.Errorf("too many ips using this approach to VIPs")
-		}
-		svcData.VirtualIps = []string{
-			fmt.Sprintf("10.244.0.%d", lastVIPIndex),
-		}
-
-		// populate virtual ports where we forgot them
-		var (
-			usedPorts = make(map[uint32]struct{})
-			next      = uint32(8080)
-		)
-		for _, sp := range svcData.Ports {
-			if sp.Protocol == pbcatalog.Protocol_PROTOCOL_MESH {
-				continue
-			}
-			if sp.VirtualPort > 0 {
-				usedPorts[sp.VirtualPort] = struct{}{}
-			}
-		}
-		for _, sp := range svcData.Ports {
-			if sp.Protocol == pbcatalog.Protocol_PROTOCOL_MESH {
-				continue
-			}
-			if sp.VirtualPort > 0 {
-				continue
-			}
-		RETRY:
-			attempt := next
-			next++
-			_, used := usedPorts[attempt]
-			if used {
-				goto RETRY
-			}
-			usedPorts[attempt] = struct{}{}
-			sp.VirtualPort = attempt
-		}
-	}
-	return nil
-}
-
 const permutedWarning = "use the disabled node kind if you want to ignore a node"
 
 func inheritAndValidateNodes(
@@ -835,7 +635,6 @@ func inheritAndValidateNodes(
 		}
 
 		if currNode.Node.Kind != node.Kind ||
-			currNode.Node.Version != node.Version ||
 			currNode.Node.Partition != node.Partition ||
 			currNode.Node.Name != node.Name ||
 			currNode.Node.Index != node.Index ||
@@ -872,7 +671,6 @@ func inheritAndValidateNodes(
 
 			if currWrk.ID != wrk.ID ||
 				currWrk.Port != wrk.Port ||
-				!maps.Equal(currWrk.Ports, wrk.Ports) ||
 				currWrk.EnvoyAdminPort != wrk.EnvoyAdminPort ||
 				currWrk.EnvoyPublicListenerPort != wrk.EnvoyPublicListenerPort ||
 				isSame(currWrk.Command, wrk.Command) != nil ||
