@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import Model, { attr } from '@ember-data/model';
 
 export const PRIMARY_KEY = 'uid';
@@ -9,6 +14,7 @@ export default class Role extends Model {
 
   @attr('string') Datacenter;
   @attr('string') Namespace;
+  @attr('string') Partition;
   @attr('string', { defaultValue: () => '' }) Name;
   @attr('string', { defaultValue: () => '' }) Description;
   @attr({ defaultValue: () => [] }) Policies;
